@@ -2,7 +2,7 @@ __author__ = "Vini Salazar"
 __license__ = "MIT"
 __maintainer__ = "Vini Salazar"
 __url__ = "https://github.com/vinisalazar/bioprov"
-__version__ = "0.1.17"
+__version__ = "0.1.22"
 
 
 """
@@ -11,8 +11,8 @@ Init module for package bioprov.
 Inherits objects from the src/ package.
 """
 
-from .src.config import config, EnvProv, BioProvDB
-from .src.files import File, SeqFile
+from .src.config import config, Environment, BioProvDB, Config
+from .src.files import File, SeqFile, Directory
 from .src.main import (
     Program,
     PresetProgram,
