@@ -2,23 +2,26 @@ __author__ = "Vini Salazar"
 __license__ = "MIT"
 __maintainer__ = "Vini Salazar"
 __url__ = "https://github.com/vinisalazar/bioprov"
-__version__ = "0.1.17"
+__version__ = "0.1.22"
 
 
 """
 Contains the File and SeqFile classes and related functions.
 """
 
-import numpy as np
+import logging
 from dataclasses import dataclass
 from pathlib import Path
+
+import numpy as np
 from Bio import SeqIO, AlignIO
+
 from bioprov.utils import (
     get_size,
     Warnings,
     serializer_filter,
     serializer,
-    file_to_sha1,
+    file_to_sha256,
     pattern_replacer,
 )
 
@@ -51,7 +54,7 @@ class File:
         self._exists = self.path.exists()
         self.size = get_size(self.path)
         self.raw_size = get_size(self.path, convert=False)
-        self._sha1 = file_to_sha1(self.path)
+        self._sha256 = file_to_sha256(self.path)
 
         # Provenance attributes
         self._entity = None
@@ -62,14 +65,20 @@ class File:
     def __str__(self):
         return self.__repr__()
 
-    @property
-    def sha1(self):
-        self._sha1 = file_to_sha1(self.path)
-        return self._sha1
+    def __add__(self, other):
+        return str(self) + other
 
-    @sha1.setter
-    def sha1(self, value):
-        self._sha1 = value  # no cover
+    def __radd__(self, other):
+        return other + str(self)
+
+    @property
+    def sha256(self):
+        self._sha256 = file_to_sha256(self.path)
+        return self._sha256
+
+    @sha256.setter
+    def sha256(self, value):
+        self._sha256 = value  # no cover
 
     @property
     def exists(self):
@@ -104,27 +113,119 @@ class File:
 
         :param old_terms: Terms to be replaced in the path.
         :param new: New term.
-        :param warnings: Whether to warn if sha1 checksum differs or file does not exist.
+        :param warnings: Whether to warn if sha256 checksum differs or file does not exist.
 
         :return: Updates self.
         """
-        old_hash, old_exists = self._sha1, self._exists
+        old_hash, old_exists = self._sha256, self._exists
         self.path = Path(pattern_replacer(str(self.path), old_terms, new))
-        # TODO: replace these print statements for logger warning/debug level
         if warnings:
             if not self.exists and old_exists:
-                print(
-                    f"Warning: file {self.path} was marked as existing but was not found."
+                logging.warning(
+                    f"File {self.path} was marked as existing but was not found."
                 )
-            if old_hash and self.sha1 != old_hash and self.exists:
-                print(
-                    f"Warning: file {self.path} previous sha1 checksum differs from the current."
+            if old_hash and self.sha256 != old_hash and self.exists:  # no cover
+                logging.warning(
+                    f"File {self.path} previous sha256 checksum differs from the current."
                 )
 
     def serializer(self):
         return serializer(self)
 
 
+class Directory:
+    """
+    Class for holding information about directories.
+    """
+
+    def __init__(self, path, tag=None):
+        self.path = Path(path).absolute()
+        self.name = self.path.stem
+        self.basename = self.path.name
+        if tag is None:
+            tag = self.name
+        self.tag = tag
+        self._exists = self.path.exists()
+
+        # Provenance attributes
+        self._entity = None
+
+    def replace_path(self, old_terms, new, warnings=False):
+        """
+        Replace the current File path.
+
+        Usually used for switching between users.
+
+        :param old_terms: Terms to be replaced in the path.
+        :param new: New term.
+        :param warnings: Whether to warn if sha256 checksum differs or file does not exist.
+
+        :return: Updates self.
+        """
+        old_exists = self._exists
+        self.path = Path(pattern_replacer(str(self.path), old_terms, new))
+        # TODO: replace these print statements for logger warning/debug level
+        if warnings:
+            if not self.exists and old_exists:
+                logging.warning(
+                    f"Warning: file {self.path} was marked as existing but was not found."
+                )
+
+    def __repr__(self):
+        return str(self.path)
+
+    def __str__(self):
+        return self.__repr__()
+
+    def __add__(self, other):
+        return str(self) + other
+
+    def __radd__(self, other):
+        return other + str(self)
+
+    @property
+    def exists(self):
+        self._exists = self.path.exists()
+        return self._exists
+
+    @exists.setter
+    def exists(self, value):
+        self._exists = value  # no cover
+
+    def serializer(self):
+        return serializer(self)
+
+    def get_subdirs(self):
+        return [i for i in self.path.iterdir() if i.is_dir()]
+
+    def get_files(self):
+        return [i for i in self.path.iterdir() if i.is_file()]
+
+    def add_files_to_object(self, object_, kind="files"):
+        """
+        Add files or subdirs in self to object_, can be either a Sample or Project.
+        :param object_: bioprov.Project or bioprov.Sample
+        :param kind: Whether to add files or subdirectories.
+        :return: Updates object_.files
+        """
+        choices = ("file", "dir", "subdir", "f", "d")
+        kind = kind.lower()
+        if kind.endswith("s"):
+            kind = kind[:-1]  # chop the 's' off
+        assert kind in choices, Warnings()["choices"](kind, choices, "kind")
+        if kind in ("f", "file"):
+            files = [File(i) for i in self.get_files()]
+            object_.add_files(files)
+        elif kind in ("dir", "subdir", "d"):
+            dirs = [Directory(i) for i in self.get_subdirs()]
+            object_.add_files(dirs)
+        else:
+            raise Exception(
+                "Could not add Directory content to object."
+                "Please specify a valid kind."
+            )
+
+
 class SeqFile(File):
     """
     Class for holding sequence file and sequence information. Inherits from File.
@@ -343,7 +444,7 @@ def seqrecordgenerator(path, format, parser="seq", warnings=False):
     :param path: Path to file.
     :param format: format to pass to SeqIO.parse().
     :param parser: Whether to import records with SeqIO (default) or AlignIO
-    :param warnings: Whether to warn if sha1 checksum differs or file does not exist.
+    :param warnings: Whether to warn if sha256 checksum differs or file does not exist.
 
     :return: A generator of SeqRecords.
     """
@@ -361,8 +462,8 @@ def seqrecordgenerator(path, format, parser="seq", warnings=False):
         return records
     except FileNotFoundError:
         if warnings:
-            print(Warnings()["not_exist"](path))
-            print(
+            logging.warning(Warnings()["not_exist"](path))
+            logging.warning(
                 "The file was loaded as a BioProv object, but it does not exist on the specified path."
             )
         return None
@@ -393,8 +494,14 @@ def deserialize_files_dict(files_dict):
                                 file[seqstats_attr_],
                             )
             else:
-                files_dict[tag] = File(file["path"], tag=file["tag"])
+                if file["path"].endswith("/"):
+                    files_dict[tag] = Directory(file["path"], tag=file["tag"])
+                else:
+                    files_dict[tag] = File(file["path"], tag=file["tag"])
             for attr_, value_ in file.items():
                 if attr_ not in ("path",):
-                    setattr(files_dict[tag], attr_, value_)
+                    try:
+                        setattr(files_dict[tag], attr_, value_)
+                    except AttributeError:
+                        pass
     return files_dict
