@@ -2,7 +2,7 @@ __author__ = "Vini Salazar"
 __license__ = "MIT"
 __maintainer__ = "Vini Salazar"
 __url__ = "https://github.com/vinisalazar/bioprov"
-__version__ = "0.1.17"
+__version__ = "0.1.22"
 
 """
 
@@ -24,23 +24,32 @@ This class also contains functions to read and write objects in JSON and tab-del
 
 import datetime
 import json
-import pandas as pd
+import logging
 import tempfile
-from bioprov import config
-from bioprov.utils import Warnings, serializer, serializer_filter, dict_to_sha1
-from bioprov.src.files import File, SeqFile, deserialize_files_dict
-from bioprov.src.config import EnvProv
+from collections import OrderedDict
 from collections import deque
-from coolname import generate_slug
 from os import path
 from pathlib import Path
 from subprocess import Popen, PIPE, getoutput
 from time import time
 from types import GeneratorType
-from collections import OrderedDict
+
+import pandas as pd
+from coolname import generate_slug
 from prov.model import ProvEntity, ProvBundle, Namespace
 from tinydb import Query
 
+from bioprov import config
+from bioprov.src.config import Environment, BioProvDB
+from bioprov.src.files import File, SeqFile, Directory, deserialize_files_dict
+from bioprov.utils import (
+    Warnings,
+    serializer,
+    serializer_filter,
+    dict_to_sha256,
+    create_logger,
+)
+
 
 class Program:
     """
@@ -55,6 +64,7 @@ class Program:
         path_to_bin=None,
         version=None,
         cmd=None,
+        sample=None,
     ):
         """
         :param name: Name of the program being called.
@@ -63,6 +73,7 @@ class Program:
         :param path_to_bin: A full _path to the program's binary. Default: get from self.name.
         :param cmd: A command string to call the program. Default: build from self._path and self.params.
         :param version: Version of the program.
+        :param sample: Instance of bioprov.Sample
         """
         self.name = name
         self.cmd = cmd
@@ -72,6 +83,7 @@ class Program:
         self.path = path_to_bin
         self.version = version
         self._getoutput = getoutput(f"which {self.name}")
+        self.sample = sample
         self.found = (
             "command not found" not in self._getoutput
             and self._getoutput != ""
@@ -98,6 +110,62 @@ class Program:
     def runs(self, value):
         self._runs = value
 
+    @property
+    def stdin(self):
+        try:
+            return self.runs[list(self.runs.keys())[-1]].stdin
+        except KeyError:
+            return None
+
+    @property
+    def stdout(self):
+        try:
+            return self.runs[list(self.runs.keys())[-1]].stdout
+        except KeyError:
+            return None
+
+    @property
+    def stderr(self):
+        try:
+            return self.runs[list(self.runs.keys())[-1]].stderr
+        except KeyError:
+            return None
+
+    @property
+    def start_time(self):
+        try:
+            return self.runs[list(self.runs.keys())[-1]].start_time
+        except KeyError:
+            return None
+
+    @property
+    def end_time(self):
+        try:
+            return self.runs[list(self.runs.keys())[-1]].end_time
+        except KeyError:
+            return None
+
+    @property
+    def duration(self):
+        try:
+            return self.runs[list(self.runs.keys())[-1]].duration
+        except KeyError:
+            return None
+
+    @property
+    def finished(self):
+        try:
+            return self.runs[list(self.runs.keys())[-1]].finished
+        except KeyError:
+            return None
+
+    @property
+    def status(self):
+        try:
+            return self.runs[list(self.runs.keys())[-1]].status
+        except KeyError:
+            return None
+
     def add_runs(self, runs):
         """
         Sample method to add runs.
@@ -116,13 +184,14 @@ class Program:
         self.cmd = cmd
         return cmd
 
-    def add_parameter(self, parameter, _print=False):
+    def add_parameter(self, parameter, _generate_cmd=True):
         """
         Adds a parameter to the current instance and updates the command.
 
         :param parameter: an instance of the Parameter class.
-        :param _print: whether to print the parameter has been added.
-        :return:
+        :param _generate_cmd: Refreshes self.cmd when a Parameter is added.
+
+        :return: Updates self.params and self.cmd if _generate_cmd is True.
         """
         assert isinstance(parameter, Parameter), Warnings()["incorrect_type"](
             parameter, Parameter
@@ -130,23 +199,48 @@ class Program:
         k, v = parameter.key, parameter.value
         self.params[k] = parameter
         self.param_str = generate_param_str(self.params)
-        self.generate_cmd()
-        if _print:
-            print(f"Added parameter {k} with value '{v}' to program {self.name}")
+        config.logger.debug(
+            f"Added parameter {k} with value '{v}' to program {self.name}"
+        )  # no cover
+        if _generate_cmd:
+            self.generate_cmd()
 
-    def run(self, sample=None, _print=True):
+    def run(
+        self,
+        sample=None,
+        suppress_stdout=True,
+        suppress_stderr=True,
+        force_print=False,
+    ):
         """
         Runs the process.
-        :param sample: An instance of the Sample class
-        :param _print: Argument to pass to Run.run()
-        :return: An instance of Run class.
+        :param sample: An instance of bioprov.Sample.
+        :param suppress_stdout: Whether to print stdout of the program.
+        :param suppress_stderr: Whether to print stderr of the program.
+        :param force_print: Whether to force printing the output of the program.
+        :return: An instance of the Run class.
         """
-
         # Creates Run instance with self
-        run_ = Run(self, sample=sample)
-        run_.run(_sample=sample, _print=True)
-        self.add_runs(run_)
-        return run_
+        if sample is None:
+            sample = self.sample
+        _run = Run(self, sample=sample)
+        _run.run()
+        self.add_runs(_run)
+
+        if _run.auto_suppress_stdout and not force_print:
+            suppress_stdout = True
+            suppress_stderr = True
+
+        if not suppress_stdout:
+            print("\n")
+            print("stdout:\n")
+            print(_run.stdout)
+        if not suppress_stderr:
+            print("\n")
+            print("stderr:\n")
+            print(_run.stderr)
+
+        return _run
 
     def serializer(self):
         keys = [
@@ -279,7 +373,7 @@ class Run:
         self.stderr = None
 
         # This parameter will suppress from writing stdout if it is too long.
-        self._auto_suppress_stdout = True
+        self.auto_suppress_stdout = True
 
         # Time status
         self.start_time = None
@@ -322,51 +416,46 @@ class Run:
         dict_ = {True: "Finished", False: "Pending"}
         return dict_[finished_status]
 
-    def run(self, _sample=None, _print=True, _print_stdout=False, _print_stderr=False):
+    def run(self):
         """
         Runs process for the Run instance.
         Will update attributes accordingly.
-        :type _print: bool
-        :param _sample: self.sample
-        :param _print_stdout: Whether to print the stdout of the Program.
-        :param _print_stderr: Whether to print the stderr of the Program.
-        :return: self.stdout
+        :return: self
         """
-        if _sample is None:
-            _sample = self.sample
 
         # Declare process and start time
         assert (
             self.program.found
         ), f"Cannot find program {self.program.name}. Make sure it is on your $PATH."
-        if _print:
-            str_ = f"Running program '{self.program.name}'"
-            if _sample is not None:
-                str_ += f" for sample {_sample.name}."
-            else:
-                str_ += "."
 
-            # Pretty printing of commands
-            split_ = self.program.cmd.split()
-            if len(self.program.cmd) > 80:
-                if len(split_) % 2 == 1:
-                    bin_, *fmt_cmd = split_
-                    last = ""
-                else:
-                    bin_, *fmt_cmd, last = split_
-                it = iter(fmt_cmd)
-                fmt_cmd = zip(it, it)
-                fmt_cmd = " \\ \n".join(
-                    [bin_] + ["\t" + i[0] + " " + i[1] for i in fmt_cmd] + ["\t" + last]
-                )
-                str_ += f"\nCommand is:\n{fmt_cmd}\n"
-            else:
-                str_ += f"\nCommand is:\n{self.program.cmd}\n"
+        # Print block
+        str_ = f"Running program '{self.program.name}'"
+        if self.sample is not None:
+            str_ += f" for sample {self.sample.name}."
+        else:
+            str_ += "."
 
-            str_ = str_.strip()
-            if str_.endswith("\\"):
-                str_ = str_[:-1]
-            print(str_)
+        # Pretty printing of commands
+        split_ = self.program.cmd.split()
+        if len(self.program.cmd) > 80:
+            if len(split_) % 2 == 1:
+                bin_, *fmt_cmd = split_
+                last = ""
+            else:
+                bin_, *fmt_cmd, last = split_  # no cover
+            it = iter(fmt_cmd)
+            fmt_cmd = zip(it, it)
+            fmt_cmd = " \\ \n".join(
+                [bin_] + ["\t" + i[0] + " " + i[1] for i in fmt_cmd] + ["\t" + last]
+            )
+            str_ += f"\nCommand is:\n{fmt_cmd}\n"
+        else:
+            str_ += f"\nCommand is:\n{self.program.cmd}\n"
+
+        str_ = str_.strip()
+        if str_.endswith("\\"):
+            str_ = str_[:-1]
+        config.logger.info(str_)
 
         p = Popen(self.program.cmd, shell=True, stdout=PIPE, stderr=PIPE)
         self.process = p
@@ -389,11 +478,9 @@ class Run:
         self.finished = True
         self._status = self._finished_to_status(self.finished)
 
-        # These are useful for quick debugging.
-        if _print_stdout:
-            print(self.stdout)
-        if _print_stderr:
-            print(self.stderr)
+        if not self.auto_suppress_stdout:
+            config.logger.debug(self.stdout)  # no cover
+        config.logger.debug(self.stderr)  # no cover
 
         return self
 
@@ -406,7 +493,7 @@ class Run:
                     if (
                         serial_out[key] is not None
                         and len(serial_out[key]) > 5000
-                        and self._auto_suppress_stdout
+                        and self.auto_suppress_stdout
                     ):
                         serial_out[key] = None
                 else:
@@ -452,6 +539,7 @@ class PresetProgram(Program):
         input_files=None,
         output_files=None,
         preffix_tag=None,
+        extra_flags=None,
     ):
         """
         :param name: Instance of bioprov.Program
@@ -467,6 +555,8 @@ class PresetProgram(Program):
         :param preffix_tag: A value in the input_files argument, which corresponds
                             to a key in self.sample.files. All file names of output
                             files will be stemmed from this file, hence 'preffix'.
+        :param extra_flags: A list of command line parameters (strings), known as flags or
+                            switches, to add to the program's command.
         """
         super().__init__(name, params)
         self.sample = sample
@@ -477,12 +567,17 @@ class PresetProgram(Program):
         self.input_files = input_files
         self.output_files = output_files
         self.preffix_tag = preffix_tag
+        self.extra_flags = extra_flags
         self.ready = False
-        self.generate_cmd()
 
         if self.sample is not None:
             self.create_func(sample=self.sample, preffix_tag=self.preffix_tag)
 
+        if self.extra_flags is not None:
+            _extra_flags = [Parameter(key=flag) for flag in self.extra_flags]
+            for _flag in _extra_flags:
+                self.add_parameter(_flag)
+
     def _parse_input_files(self):
         """
         Checks if input files exist and adds them to self.program.
@@ -494,7 +589,8 @@ class PresetProgram(Program):
                 file_ = self.sample.files[tag]
             except KeyError:
                 raise Exception(
-                    f"Key '{tag}' not found in files dictionary of sample '{self.sample.name}':\n'{self.sample.files}'"
+                    f"Key '{tag}' not found in files dictionary of sample '{self.sample.name}':\n'{self.sample.files}'."
+                    f"\nPlease specify a valid input tag."
                 )
 
             # If in sample, check if it exists
@@ -504,7 +600,7 @@ class PresetProgram(Program):
             param = Parameter(
                 key=k, value=str(self.sample.files[tag]), kind="input", tag=tag
             )
-            self.add_parameter(param)
+            self.add_parameter(param, _generate_cmd=False)
 
     def _parse_output_files(self):
         """
@@ -524,12 +620,23 @@ class PresetProgram(Program):
                     f"'{self.sample.files}'"
                 )
         try:
-            for key, (tag, suffix) in self.output_files.items():
-                self.sample.add_files({tag: preffix + suffix})
+            # TODO: refactor the `input_files` and `output_files` parameter as NamedTuples
+            # This will also allow us to specify directories
+            for key, value in self.output_files.items():
+                # Usually just specify tag and suffix
+                if len(value) == 2:
+                    suffix, tag = value
+                    self.sample.add_files(File(preffix + suffix, tag=tag))
+                # But we can also specify a format
+                elif len(value) == 3:
+                    suffix, tag, format = value
+                    self.sample.add_files(
+                        SeqFile(preffix + suffix, tag=tag, format=format)
+                    )
                 param = Parameter(
                     key=key, value=str(self.sample.files[tag]), kind="output", tag=tag
                 )
-                self.add_parameter(param, _print=False)
+                self.add_parameter(param, _generate_cmd=False)
         except ValueError:
             raise Exception(
                 "Please check the output files dictionary:\n'{}'\n"
@@ -557,8 +664,12 @@ class PresetProgram(Program):
         self._parse_input_files()
         self._parse_output_files()
 
+        # Add self to sample automatically
+        self.sample.add_programs(self)
+
         # Set ready state
         self.ready = True
+        self.generate_cmd()
 
     def validate_sample(self):
         """
@@ -576,13 +687,11 @@ class PresetProgram(Program):
         """
         assert isinstance(self, Program), Warnings()["incorrect_type"](self, Program)
 
-    def generate_cmd(self, from_files=True):
+    def generate_cmd(self):
         """
         TODO: improve this function
 
         Generates a wildcard command string, independent of samples.
-        :param from_files: Generate command from self.input_files and self.output_files (recommended) If False,
-        will generate from parameter dictionary instead.
         :return: Updates self.cmd.
         """
         self.validate_program()
@@ -595,7 +704,7 @@ class PresetProgram(Program):
                 try:
                     parameter.value = str(self.sample.files[f"{parameter.tag}"])
                 except AttributeError:
-                    print("Warning: no sample associated with program.")
+                    config.logger.warning("Warning: no sample associated with program.")
                     pass  # Suppress bug for now.
             else:
                 pass
@@ -606,23 +715,21 @@ class PresetProgram(Program):
         self.cmd = generic_cmd
         return generic_cmd
 
-    def run(self, sample=None, _print=True, preffix_tag=None):
+    def run(self, sample=None, preffix_tag=None, **kwargs):
         """
         Runs PresetProgram for sample.
         :param sample: Instance of bioprov.Sample.
-        :param _print: Whether to print more output.
         :param preffix_tag: Preffix tag to self.create_func()
+        :param kwargs: See help of Program.run()
         :return:
         """
         if sample is None:
             sample = self.sample
         if preffix_tag is None:
             preffix_tag = self.preffix_tag
-        if not self.ready:
-            self.create_func(sample, preffix_tag)
 
-        # Update self._run, run self.run() and update self._run again.
-        Program.run(self, sample=sample, _print=_print)
+        self.create_func(sample, preffix_tag)
+        return super().run(**kwargs)
 
 
 def parse_params(params):
@@ -674,7 +781,9 @@ def generate_param_str(params):
         param_str = str_.strip()
     else:
         # TODO: add more parameters options. List of tuples, List of Parameter instances, etc.
-        print("Must provide either a string or a dictionary for the parameters!")
+        config.logger.error(
+            "Must provide either a string or a dictionary for the parameters!"
+        )
         raise TypeError
     # Add positional arguments
     split_str = param_str.split()
@@ -716,6 +825,8 @@ def _add_programs(object_, programs):
     for program in programs:
         object_.programs[program.name] = program
 
+    object_.auto_update_db()
+
 
 def _add_runs(object_, runs):
     """
@@ -757,11 +868,14 @@ class Sample:
     Class for holding sample information and related files and programs.
     """
 
-    def __init__(self, name=None, tag=None, files=None, attributes=None):
+    def __init__(
+        self, name=None, tag=None, files=None, directory=None, attributes=None
+    ):
         """
         :param name:  Sample name or ID.
         :param tag: optional tag describing the sample.
         :param files: Dictionary of files associated with the sample.
+        :param directory: A bioprov.Directory associated with the sample
         :param attributes: Dictionary of any other attributes associated with the sample.
         """
         if isinstance(name, str):
@@ -780,23 +894,32 @@ class Sample:
         elif files is None:
             files = dict()
         self.files = files
+        self._directory = directory
         if attributes is not None:
             assert isinstance(attributes, dict)
         else:
             attributes = dict()
         self.attributes = attributes
         self.programs = OrderedDict()
+        self.project = None
 
         # This is an attribute used by the src.prov module
         self.namespace_preffix = f"samples:{self.name}"
         self.files_namespace_preffix = None
 
+    def auto_update_db(self):
+        if self.project is not None:
+            self.project.auto_update_db()
+
     def __repr__(self):
-        str_ = f"Sample {self.name} with {len(self.files)} file(s)."
+        str_ = f"Sample '{self.name}' with {len(self.files)} file(s)."
         return str_
 
-    def __getitem__(self, item):
-        return self.files[item]
+    def __getitem__(self, key):
+        return self.files[key]
+
+    def __delitem__(self, key):
+        del self.files[key]
 
     def __setitem__(self, key, value):
         assert isinstance(
@@ -804,6 +927,26 @@ class Sample:
         ), f"To create file in sample, must be either a bioprov.File or bioprov.SequenceFile instance."
         self.files[key] = value
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        return
+
+    @property
+    def directory(self):
+        if self._directory is None and self.files:
+            key, file = next(iter(self.files.items()))
+            directory = Directory(file.directory, tag=key)
+            self._directory = directory
+        return self._directory
+
+    @directory.setter
+    def directory(self, value):
+        if isinstance(value, (str, Path)):
+            value = Directory(value)
+        self._directory = value
+
     def add_programs(self, programs):
         """
         Adds program(s) to self. Must be an instance or iterable of bioprov.Program.
@@ -826,33 +969,31 @@ class Sample:
         Custom serializer for Sample class. Serializes runs, programs, and files attributes.
         :return:
         """
-        keys = ["files_namespace_preffix"]
+        keys = ["files_namespace_preffix", "project"]
         return serializer_filter(self, keys)
 
-    def run_programs(self, _print=True):
+    def run_programs(self):
         """
         Runs self._programs in order.
         :return:
         """
-        _run_programs(self, _print)
+        _run_programs(self)
 
-    def _run_program(self, program, _print=True):
+    def _run_program(self, program):
         """
         Runs program for self.
         :param program: bioprov.Program or bioprov.PresetProgram.
-        :param _print: Whether to print output of Program.
         :return: Runs Program and updates self.
         """
-        _run_program(self, program, _print)
+        _run_program(self, program)
 
-    def to_json(self, _path=None, _print=True):
+    def to_json(self, _path=None):
         """
         Exports the Sample as JSON. Similar to Project.to_json()
         :param _path: JSON output file path.
-        :param _print: Whether to print if the file was created correctly.
         :return:
         """
-        return to_json(self, self.serializer(), _path, _print=_print)
+        return to_json(self, self.serializer(), _path)
 
     def to_series(self):
         """
@@ -863,7 +1004,7 @@ class Sample:
         series = {}
 
         # Can't apply serializer_filter here.
-        keys = ["files_namespace_preffix", "namespace_preffix", "_programs"]
+        keys = ["files_namespace_preffix", "namespace_preffix", "_programs", "project"]
         modified_dict = self.__dict__.copy()
         for key in keys:
             try:
@@ -883,34 +1024,33 @@ class Sample:
         return pd.Series(series)
 
 
-def _run_program(_object, program, _print=True):
+def _run_program(_object, program):
     """
     Runs program for _object.
 
     :param _object: bioprov.Project or bioprov.Sample
     :param program: bioprov.Program or bioprov.PresetProgram.
-    :param _print: Whether to print output of Program.
     :return: Runs Program and updates _object.
     """
     if program not in _object.programs.keys():
         _object.add_programs(program)
-    program.run(sample=_object, _print=_print)
+    program.run(sample=_object)
+    _object.auto_update_db()
 
 
-def _run_programs(_object, _print=True):
+def _run_programs(_object):
     """
     Runs programs in order.
 
     :param _object: bioprov.Project or bioprov.Sample
-    :param _print: Whether to print output of Program.
     :return:
     """
     if len(_object.programs) >= 1:
         for _, p in _object.programs.items():
             # noinspection PyProtectedMember
-            _object._run_program(p, _print=_print)
+            _object._run_program(p)
     else:
-        print(f"No programs to run for {_object}")
+        config.logger.warning(f"No programs to run for {_object}")
 
 
 class Project:
@@ -918,7 +1058,14 @@ class Project:
     Class which holds a dictionary of Sample instances, where each key is the sample name.
     """
 
-    def __init__(self, samples=None, tag=None, db=None, auto_update=False):
+    def __init__(
+        self,
+        tag=None,
+        samples=None,
+        db=None,
+        auto_update=False,
+        log_to_file=False,
+    ):
         """
         Initiates the object by creating a sample dictionary.
         :param samples: An iterator of Sample objects.
@@ -926,9 +1073,15 @@ class Project:
         :param db: path to TinyDB to store project in JSON format.
         :param auto_update: Whether to auto_update the BioProvDB record.
                             Disabled by default.
+        :param log_to_file: Whether to log the Project to a File. You can define this later with
+                            the self.start_logging() method.
         """
         if tag is None:
-            tag = generate_slug(2)
+            slug = generate_slug(2)
+            config.logger.warning(
+                f"No Project tag was set. Generated random tag: '{slug}'."
+            )
+            tag = slug
         self.tag = tag.replace(" ", "_")
         self._name = self.tag
         self.files = dict()
@@ -946,30 +1099,44 @@ class Project:
         # PROV attributes
         self._entity = None
         self._bundle = None
-        self.namespace_preffix = f"project:{self}"
+        self._namespace_preffix = f"project:{self.tag}"
         self.files_namespace_preffix = None
 
         # Hash and db attributes
-        self._sha1 = dict_to_sha1(self.serializer())
+        self._sha256 = dict_to_sha256(self.serializer())
         self.auto_update = auto_update
         if db is None:
             db = config.db
         self.db = db
 
+        # Log attributes
+        self.log_to_file = log_to_file
+        self.log_file = None
+        self.logger = None
+        if self.log_to_file:
+            self.start_logging()
+
     def __len__(self):
         return len(self._samples)
 
     def __repr__(self):
-        return f"Project '{self.tag}' with {len(self._samples)} samples."
+        return f"Project '{self.tag}' with {len(self)} samples"
 
     def __getitem__(self, item):
-        try:
-            value = self._samples[item]
-            return value
-        except KeyError:
-            print(f"Sample {item} not in Project.\n")
-            print("Check the following keys:")
-            print(" ", "\n  ".join(self.keys))
+        if isinstance(item, str):
+            try:
+                value = self._samples[item]
+                return value
+            except KeyError:
+                config.logger.error(
+                    f"Sample {item} not in Project.\n" f"Check the following keys:" " ",
+                    "\n  ".join(list(self.keys)),
+                )
+        elif isinstance(item, int):
+            return self[list(self._samples.keys())[item]]
+
+    def __iter__(self):
+        return iter(self._samples.values())
 
     def __setitem__(self, key, value):
         self._samples[key] = value
@@ -977,6 +1144,11 @@ class Project:
     def __delitem__(self, key):
         del self._samples[key]
 
+    @property
+    def namespace_preffix(self):
+        self._namespace_preffix = f"project:{str(self)}"
+        return self._namespace_preffix
+
     def keys(self):
         return self._samples.keys()
 
@@ -992,17 +1164,17 @@ class Project:
         return self._name
 
     @property
-    def sha1(self):
-        keys = ("_sha1",)
-        new_hash = dict_to_sha1(serializer_filter(self, keys))
-        if new_hash != self._sha1:
-            self._sha1 = new_hash
+    def sha256(self):
+        keys = ("_sha256",)
+        new_hash = dict_to_sha256(serializer_filter(self, keys))
+        if new_hash != self._sha256:
+            self._sha256 = new_hash
             self.auto_update_db()
-        return self._sha1
+        return self._sha256
 
-    @sha1.setter
-    def sha1(self, value):
-        self._sha1 = value
+    @sha256.setter
+    def sha256(self, value):
+        self._sha256 = value
 
     @property
     def entity(self):
@@ -1029,10 +1201,9 @@ class Project:
             db = self.db
         result, query = self.query_db(db)
         if result:
-            print(f"Updating project '{self.tag}' at {db.db_path}")
             db.update(self.serializer(), query.tag == self.tag)
         else:
-            print(f"Inserting new project '{self.tag}' in {db.db_path}")
+            config.logger.info(f"Inserting new project '{self.tag}' in {db.db_path}")
             db.insert(self.serializer())
 
     def auto_update_db(self):
@@ -1074,6 +1245,30 @@ class Project:
             for _, file in sample.files.items():
                 file.replace_path(old_terms, new, warnings)
 
+    def start_logging(
+        self, log_file=None, level=logging.INFO, _custom_start_message=None
+    ):
+        """
+        Starts logging Project to File.
+
+        :param log_file: Path to log file. If None will be defined automatically.
+        :param level: Logging level.
+        :param _custom_start_message: Custom starting message to start the log.
+        :return: Creates logger attributes and refreshes bp.config.logger
+        """
+        if log_file is None:
+            log_file = f"{self.tag}.log"
+        self.log_file = log_file
+        self.logger = config.logger = create_logger(level, self.log_file, self.tag)
+        if _custom_start_message is None:
+            _custom_start_message = f"Starting log for project '{self.tag}'."
+
+        self.logger.info(_custom_start_message)
+
+        self.add_files(File(self.log_file, tag="log"))
+        self.logger.info(f"Writing log to {self.files['log']}")
+        self.logger.info(f"Loading {len(self)} samples.")
+
     def add_files(self, files):
         """
         Adds Files to self.files. See documentation to bioprov.src.main.add_files().
@@ -1082,7 +1277,6 @@ class Project:
         :return: Updates self.files
         """
         _add_files(self, files)
-        self.auto_update_db()
 
     def add_programs(self, programs):
         """
@@ -1091,25 +1285,23 @@ class Project:
         :return: Updates self.programs
         """
         _add_programs(self, programs)
-        self.auto_update_db()
 
-    def run_programs(self, _print=True):
+    def run_programs(self):
         """
         Runs all programs in self.programs in order.
 
         :return:
         """
-        _run_programs(self, _print)
+        _run_programs(self)
 
-    def _run_program(self, program=None, _print=True):
+    def _run_program(self, program=None):
         """
         Runs program for self.
 
         :param program: bioprov.Program or bioprov.PresetProgram.
-        :param _print: Whether to print output of Program.
         :return: Runs Program and updates self.
         """
-        _run_program(self, program, _print)
+        _run_program(self, program)
 
     @staticmethod
     def is_sample_and_name(sample):
@@ -1126,7 +1318,9 @@ class Project:
         if sample.name is None:
             slug = generate_slug(2)
             sample.name = slug
-            print(f"No sample name set. Setting random name: {sample.name}")
+            config.logger.warning(
+                f"No sample name set. Setting random name: {sample.name}"
+            )
 
         return sample
 
@@ -1147,8 +1341,7 @@ class Project:
 
         return constructor
 
-    @staticmethod
-    def build_sample_dict(constructor):
+    def build_sample_dict(self, constructor):
         """
         Build sample dictionary from passed constructor.
         :param constructor: Iterable or NoneType
@@ -1164,6 +1357,7 @@ class Project:
         for sample in constructor:
             sample = Project.is_sample_and_name(sample)
             samples[sample.name] = sample
+            samples[sample.name].project = self
 
         return samples
 
@@ -1178,14 +1372,13 @@ class Project:
     def serializer(self):
         return serializer(self)
 
-    def to_json(self, _path=None, _print=True):
+    def to_json(self, _path=None):
         """
         Exports the Project as JSON. Similar to Sample.to_json()
         :param _path: JSON output file _path.
-        :param _print: Whether to print if the file was created correctly.
         :return:
         """
-        return to_json(self, self.serializer(), _path, _print=_print)
+        return to_json(self, self.serializer(), _path)
 
     def to_df(self):
         """
@@ -1243,11 +1436,20 @@ def _add_files(object_, files):
     if isinstance(files, dict):
         files_ = dict()
         for k, v in files.items():
+
             # This is to convert JSON files.
             if isinstance(v, dict):
-                files_[k] = File(v["path"], v["tag"])
+                if v["path"].endswith("/"):
+                    files_[k] = Directory(v["path"], v["tag"])
+                else:
+                    files_[k] = File(v["path"], v["tag"])
+
+            # This is the usual flow
             else:
-                files_[k] = File(v, tag=k)
+                if str(v).endswith("/"):
+                    files_[k] = Directory(v, tag=k)
+                else:
+                    files_[k] = File(v, tag=k)
 
         files = files_
 
@@ -1256,17 +1458,19 @@ def _add_files(object_, files):
         files = {file.name: file for file in files}
 
     # If it is a single item, also transform to dict
-    elif isinstance(files, File):
+    elif isinstance(files, (File, Directory)):
         files = {files.tag: files}  # Grabs by tag because it is File.name by default
 
-    # Here files must be a dictionary of File instances
+    # Here 'files' must be a dictionary of File or Directory instances
     for k, v in files.items():
         if k in object_.files.keys():
-            print(f"Updating file {k} with value {v}.")
+            config.logger.info(f"Updating file {k} with value {v}.")
         object_.files[k] = v
 
+    object_.auto_update_db()
 
-def to_json(object_, dictionary, _path=None, _print=True):
+
+def to_json(object_, dictionary, _path=None):
     """
     Exports the Sample or Project as JSON.
     :return: Writes JSON output
@@ -1278,7 +1482,7 @@ def to_json(object_, dictionary, _path=None, _print=True):
     if "json" not in object_.files.keys():
         object_.add_files({"json": _path})
 
-    return write_json(dictionary, _path, _print=_print)
+    return write_json(dictionary, _path)
 
 
 def from_json(json_file, kind="Project", replace_path=None, replace_home=False):
@@ -1334,7 +1538,7 @@ def from_json(json_file, kind="Project", replace_path=None, replace_home=False):
             pass
 
         # Create Project
-        project = Project(samples=samples, tag=d["tag"])
+        project = Project(tag=d["tag"], samples=samples)
 
         # Deserializing and adding project files and programs
         try:
@@ -1354,10 +1558,10 @@ def from_json(json_file, kind="Project", replace_path=None, replace_home=False):
             for user, env in d["users"].items():
                 for env_hash, env_dict in env.items():
                     try:
-                        project.users[user][env_hash] = EnvProv()
+                        project.users[user][env_hash] = Environment()
                     except KeyError:
                         project.users[user] = dict()
-                        project.users[user][env_hash] = EnvProv()
+                        project.users[user][env_hash] = Environment()
                     for env_attr_, attr_value_ in env_dict.items():
                         if env_attr_ == "env_namespace":
                             attr_value_ = Namespace(
@@ -1374,9 +1578,11 @@ def from_json(json_file, kind="Project", replace_path=None, replace_home=False):
             project.replace_paths(other_HOME_variables, HOME, warnings=True)
 
         if replace_path:
-            print("Replacing paths:")
-            print(f"\tOld:\t{replace_path[0][0]}")
-            print(f"\tNew:\t{replace_path[1]}")
+            config.logger.info(
+                "Replacing paths:"
+                f"\tOld:\t{replace_path[0][0]}"
+                f"\tNew:\t{replace_path[1]}"
+            )
             project.replace_paths(replace_path[0], replace_path[1], warnings=True)
 
         return project
@@ -1448,7 +1654,7 @@ def from_df(
                 sample.attributes[attr_] = row[attr_]
         samples[ix] = sample
 
-    samples = Project(samples, tag=tag)
+    samples = Project(tag=tag, samples=samples)
     if source_file:
         samples.add_files({"project_csv": source_file})
 
@@ -1516,40 +1722,44 @@ def dict_to_sample(json_dict):
     return sample_
 
 
-def write_json(dict_, _path, _print=True):
+def write_json(dict_, _path):
     """
     Writes dictionary to JSON file.
     :param dict_: JSON dictionary.
     :param _path: String with _path to JSON file.
-    :param _print: Whether to print if the file was successfully created.
     :return: Writes JSON file.
     """
     with open(_path, "w") as f:
         json.dump(dict_, f, indent=3)
 
-    if _print:
-        if Path(_path).exists():
-            print(f"Created JSON file at {_path}.")
-        else:
-            print(f"Could not create JSON file for {_path}.")
+    if Path(_path).exists():
+        config.logger.info(f"Created JSON file at {_path}.")
+    else:
+        config.logger.info(f"Could not create JSON file for {_path}.")
 
 
-def load_project(tag):
+def load_project(tag, db=None):
     """
     Loads Project from the BioProvDatabase set in the config.
 
     :param tag: Tag of the Project to be loaded.
+    :param db: Path to BioProvDB file. Default is set in the config module. (use the `bioprov --show_db` command).
     :return: Instance of Project.
     """
-    assert (
-        len(config.db) > 0
-    ), f"Project not found. Database at '{config.db_path}' is empty"
+    if db is None:
+        db = config.db
+
+    else:
+        assert Path(db).exists(), Warnings()["not_exist"](db)
+        db = BioProvDB(path=db)
+
+    assert len(db) > 0, f"Project not found. Database at '{db.db_path}' is empty"
 
     query = Query()
     try:
-        result = config.db.search(query.tag == tag)[0]
+        result = db.search(query.tag == tag)[0]
     except (IndexError, KeyError):
-        print(f"Project not found in database at {config.db_path}")
+        config.logger.error(f"Project not found in database at {db.db_path}")
         return
 
     with tempfile.NamedTemporaryFile() as f:
