@@ -2,7 +2,7 @@ __author__ = "Vini Salazar"
 __license__ = "MIT"
 __maintainer__ = "Vini Salazar"
 __url__ = "https://github.com/vinisalazar/bioprov"
-__version__ = "0.1.17"
+__version__ = "0.1.22"
 
 
 """
@@ -10,9 +10,11 @@ Integration testing for drafting new ideas.
 """
 
 from argparse import Namespace
-import bioprov as bp
-import pytest
 from os import remove
+
+import pytest
+
+import bioprov as bp
 from bioprov.data import picocyano_dataset
 
 
@@ -80,19 +82,25 @@ def test_CLI():
     assert pytest_wrapped_e.type == SystemExit
 
     # Test other arguments
-    args = Namespace(show_config=True, show_db=False, version=False, list=False)
+    args = Namespace(
+        show_config=True, show_db=False, version=False, list=False, show_provstore=False
+    )
     with pytest.raises(SystemExit) as pytest_wrapped_e:
         main(args)
 
     assert pytest_wrapped_e.type == SystemExit
 
-    args = Namespace(show_config=False, show_db=True, version=False, list=False)
+    args = Namespace(
+        show_config=False, show_db=True, version=False, list=False, show_provstore=False
+    )
     with pytest.raises(SystemExit) as pytest_wrapped_e:
         main(args)
 
     assert pytest_wrapped_e.type == SystemExit
 
-    args = Namespace(show_config=False, show_db=False, version=True, list=False)
+    args = Namespace(
+        show_config=False, show_db=False, version=True, list=False, show_provstore=False
+    )
     with pytest.raises(SystemExit) as pytest_wrapped_e:
         main(args)
 
@@ -104,13 +112,25 @@ def test_CLI():
         show_db=False,
         version=False,
         list=False,
+        show_provstore=False,
     )
-    with pytest.raises(AttributeError) as pytest_wrapped_e:
+
+    with pytest.raises(KeyError) as pytest_wrapped_e:
         main(args)
 
-    assert pytest_wrapped_e.type == AttributeError
+    assert pytest_wrapped_e.type == KeyError
 
-    args = Namespace(show_config=False, show_db=False, version=False, list=True)
+    args = Namespace(
+        show_config=False, show_db=False, version=False, list=True, show_provstore=False
+    )
+    with pytest.raises(SystemExit) as pytest_wrapped_e:
+        main(args)
+
+    assert pytest_wrapped_e.type == SystemExit
+
+    args = Namespace(
+        show_config=False, show_db=False, version=False, list=True, show_provstore=True
+    )
     with pytest.raises(SystemExit) as pytest_wrapped_e:
         main(args)
 
