@@ -2,22 +2,24 @@ __author__ = "Vini Salazar"
 __license__ = "MIT"
 __maintainer__ = "Vini Salazar"
 __url__ = "https://github.com/vinisalazar/bioprov"
-__version__ = "0.1.17"
+__version__ = "0.1.22"
 
 
 """
 Testing for the File module.
 """
-import bioprov as bp
-from bioprov import File, SeqFile, utils
-from bioprov.src.files import seqrecordgenerator
-from coolname import generate_slug
 from pathlib import Path
-from bioprov.data import synechococcus_genome
+
+from coolname import generate_slug
 from prov.model import ProvEntity
 
+import bioprov as bp
+from bioprov import File, SeqFile, Directory, utils
+from bioprov.data import synechococcus_genome
+from bioprov.src.files import seqrecordgenerator
 
-def test_File():
+
+def test_File_and_Directory():
     """
     Tests objects in the File module:
         - existing File instance
@@ -29,6 +31,7 @@ def test_File():
     # Test existing file
     file, tag = bp.__file__, "Init file for BioProv."
     f = File(file, tag)
+    d = Directory(f.path.parent)
     non_existing = generate_slug(2)
     nf = File("./" + non_existing)
     attributes = {
@@ -40,6 +43,7 @@ def test_File():
         "tag": f.tag == tag,
         "exists": f.exists is True,
         "repr": f.__repr__() == str(f.path),
+        "Directory_exists": d.exists,
         # Non existing file
         "non_existing": nf.exists is False,
         "no_size": nf.size == 0,
@@ -54,10 +58,11 @@ def test_File():
     for k, statement in attributes.items():
         assert statement, f"{k} did not pass!"
 
+    d.replace_path("", "")
     # test hashes
     nf.exists = True
     nf.replace_path("", "", warnings=True)  # no cover
-    nf.sha1 = generate_slug(2)
+    nf.sha256 = generate_slug(2)
     # nf.replace_path(non_existing, bp.__file__)  # no cover
     _ = f.entity
     f.entity = ProvEntity(None, generate_slug(2))
