@@ -2,7 +2,7 @@ __author__ = "Vini Salazar"
 __license__ = "MIT"
 __maintainer__ = "Vini Salazar"
 __url__ = "https://github.com/vinisalazar/bioprov"
-__version__ = "0.1.17"
+__version__ = "0.1.22"
 
 
 """
@@ -11,11 +11,14 @@ Testing for prov module.
 """
 
 from os import environ
+
+from pydot import Dot
+
 from bioprov import read_csv
 from bioprov.data import picocyano_dataset
+from bioprov.src.config import Environment
 from bioprov.src.prov import BioProvDocument
-from bioprov.src.config import EnvProv
-from bioprov.utils import dict_to_sha1
+from bioprov.utils import dict_to_sha256
 
 project = read_csv(
     picocyano_dataset, sequencefile_cols="assembly", tag="picocyanobacteria"
@@ -24,13 +27,13 @@ project = read_csv(
 
 def test_EnvProv():
     """
-    Tests the construction of an instance of EnvProv.
+    Tests the construction of an instance of Environment.
     :return:
     """
-    env = EnvProv()
+    env = Environment()
     for statement in (
         env.env_dict == dict(environ.items()),
-        env.env_hash == dict_to_sha1(env.env_dict),
+        env.env_hash == dict_to_sha256(env.env_dict),
     ):
         assert statement
 
@@ -42,4 +45,11 @@ def test_BioProvDocument():
     """
     # The BioProvDocument constructor with add_attributes=True
     # is tested in the test_src_main.py module
-    _ = BioProvDocument(project)
+    prov = BioProvDocument(project)
+
+    # __repr__
+    assert str(prov).startswith("BioProvDocument")
+
+    # dot property
+    assert type(prov.dot) == Dot
+    prov.dot = Dot()
