@@ -2,16 +2,18 @@ __author__ = "Vini Salazar"
 __license__ = "MIT"
 __maintainer__ = "Vini Salazar"
 __url__ = "https://github.com/vinisalazar/bioprov"
-__version__ = "0.1.17"
+__version__ = "0.1.22"
 
 """
 Helper functions.
 """
-import io
-import sys
-import json
 import hashlib
+import io
+import json
+import logging
+import sys
 from pathlib import Path
+
 from prov.model import Namespace, QualifiedName
 
 
@@ -46,15 +48,15 @@ def get_size(path, convert=True):
         return 0
 
 
-def file_to_sha1(path):
+def file_to_sha256(path):
     """
-    Get the sha1 of file. Returns None if it does not exist.
+    Get the sha256 of file. Returns None if it does not exist.
     :param path: A valid file path.
-    :return: sha1 digest or None.
+    :return: sha256 digest or None.
     """
     path = Path(path)
     if path.exists():
-        sha1 = hashlib.sha1()
+        sha256 = hashlib.sha256()
         # https://stackoverflow.com/questions/22058048/hashing-a-file-in-python
         BUF_SIZE = io.DEFAULT_BUFFER_SIZE
         with open(path, "rb") as f:
@@ -62,8 +64,8 @@ def file_to_sha1(path):
                 data = f.read(BUF_SIZE)
                 if not data:
                     break
-                sha1.update(data)
-        return sha1.hexdigest()
+                sha256.update(data)
+        return sha256.hexdigest()
     else:
         return None
 
@@ -174,7 +176,7 @@ def build_prov_attributes(dictionary, namespace):
     attributes = []
     for k, v in dictionary.items():
         if k == "namespace":
-            continue
+            continue  # no cover
         else:
             if not is_serializable_type(v):
                 v = str(v)
@@ -211,13 +213,13 @@ def serializer_filter(_object, keys):
     return serializer(serial_out)
 
 
-def dict_to_sha1(dictionary):
+def dict_to_sha256(dictionary):
     """
     Get sha256 hexdigest from a dictionary
     :param dictionary: dict
     :return: hexdigest
     """
-    sh = hashlib.sha1(json.dumps(dictionary).encode("utf-8"))
+    sh = hashlib.sha256(json.dumps(dictionary).encode("utf-8"))
     digest = sh.hexdigest()
     return digest
 
@@ -237,3 +239,25 @@ def pattern_replacer(pattern, iterable_of_olds, new):
         pattern = pattern.replace(old, new)
 
     return pattern
+
+
+def create_logger(log_level=logging.INFO, log_file=None, tag=None):
+    if tag is None:
+        tag = "bioprov"
+    logger = logging.getLogger(tag)
+    logger.setLevel(log_level)
+
+    # Console handler
+    stream_handler = logging.StreamHandler()
+    simple_fmt = logging.Formatter("%(message)s")
+    stream_handler.setFormatter(simple_fmt)
+    logger.addHandler(stream_handler)
+
+    # File handler
+    if log_file:
+        timestamp_fmt = logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
+        fh_handler = logging.FileHandler(log_file, mode="w", delay=True)
+        fh_handler.setFormatter(timestamp_fmt)
+        logger.addHandler(fh_handler)
+
+    return logger
